package cn.bunny.service.service.impl;

import cn.bunny.entity.order.Order;
import cn.bunny.service.mapper.OrderMapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.util.DigestUtils;

import java.math.BigDecimal;
import java.util.List;

@SpringBootTest
public class OrderServiceShardingTest {
    @Autowired
    private OrderMapper orderMapper;
    @Autowired
    private JdbcTemplate jdbcTemplate;

    // 水平分片测试
    @Test
    void testShardingOrder() {
        double random = Math.random();
        String md5DigestAsHex = DigestUtils.md5DigestAsHex(String.valueOf(random).getBytes());

        Order order = new Order();
        order.setOrderNo(md5DigestAsHex.substring(0, 6));
        order.setUserId(1L);
        order.setAmount(new BigDecimal(100));

        orderMapper.insert(order);
    }

    // 分库测试
    @Test
    void testInsertOrderStrategy() {
        for (long i = 0; i < 4; i++) {
            double random = Math.random();
            String md5DigestAsHex = DigestUtils.md5DigestAsHex(String.valueOf(random).getBytes());

            Order order = new Order();
            order.setOrderNo(md5DigestAsHex.substring(0, 6));
            order.setUserId(i);
            order.setAmount(new BigDecimal(100));

            orderMapper.insert(order);
        }
    }


    // 分库测试
    @Test
    void testInsertOrderStrategy2() {

        for (long i = 4; i < 8; i++) {
            double random = Math.random();
            String md5DigestAsHex = DigestUtils.md5DigestAsHex(String.valueOf(random).getBytes());

            Order order = new Order();
            order.setOrderNo(md5DigestAsHex.substring(0, 6));
            order.setUserId(i);
            order.setAmount(new BigDecimal(100));

            orderMapper.insert(order);
        }
    }

    // 分库分别测试，数据库必须要有这个表 生成规则 order_${0..1}
    @Test
    void testInsertOrderStrategy3() {
        // jdbcTemplate.execute("CREATE TABLE t_order (\n" +
        //         "  id BIGINT,\n" +
        //         "  order_no VARCHAR(30),\n" +
        //         "  user_id BIGINT,\n" +
        //         "  amount DECIMAL(10,2),\n" +
        //         "  PRIMARY KEY(id) USING BTREE\n" +
        //         ") ENGINE=InnoDB DEFAULT CHARSET=utf8 ROW_FORMAT=DYNAMIC;");

        for (long i = 5; i < 9; i++) {
            Order order = new Order();
            order.setOrderNo("ATGUIGU" + i);
            order.setUserId(2L);
            order.setAmount(new BigDecimal(100));
            orderMapper.insert(order);
        }
    }

    // 会查询所有将数据库自动组装
    @Test
    void selectAllOrder1() {
        List<Order> orders = orderMapper.selectList(null);
        orders.forEach(System.out::println);
    }

    // 会根据配置规则吗，比如UserId为奇数只会查询配置为奇数的数据库
    @Test
    void selectAllOrder2() {
        List<Order> orders = orderMapper.selectList(Wrappers.<Order>lambdaQuery().eq(Order::getUserId, 1L));
        orders.forEach(System.out::println);
    }
}

// UUID 不做主键因为无序的，
// 雪花算法是Twitter开源的
// 有序性：雪花算法生成的ID是有序的，其中包含时间戳信息，因此在一定程度上可以保持生成的ID的有序性，便于数据库索引等操作。
// 提高索引查询效率
// 性能：UUID是128位的，相比之下，雪花算法生成的ID长度较短（64位），在存储和索引上更加高效。
//
// 可读性：雪花算法生成的ID相对于UUID来说，更容易解析和理解，因为其中包含了时间戳等信息，可以直观地看出ID的生成时间。
//
// 分布式特性：雪花算法考虑了分布式系统中的唯一性要求，通过机器ID和序列号等信息，保证在分布式环境下生成的ID的唯一性。